from sqlite3 import Error
import click
from .add_by_file import add_helper
from .api_wrapper import request_data


@click.command(help="Add an asset to Tenable.io from another source via CLI")
@click.option('--ip', default='', help="IP address of the new asset")
@click.option('--mac', default='', help="Mac Address of the new asset")
@click.option('--netbios', default='', help="NetBios of the new asset")
@click.option('--fqdn', default='', help='FQDN of the new asset')
@click.option('--hostname', default='', help="Hostname of the new asset")
@click.option('--file', default='', help="Provide a CSV file in this order: IP, MAC, FQDN, Hostname. Leave fields blank if N/A")
@click.option('--source', default='navi', help="Provide the source of the information")
def add(ip, mac, netbios, fqdn, hostname, file, source):
    try:
        asset = {}
        ipv4 = []
        macs = []
        fqdns = []
        hostnames = []
        if ip:
            ipv4.append(ip)
            asset["ip_address"] = ipv4

        if mac:
            macs.append(mac)
            asset["mac_address"] = macs

        if netbios:
            asset["netbios_name"] = netbios

        if fqdn:
            fqdns.append(fqdn)
            asset["fqdn"] = fqdns

        if hostname:
            hostnames.append(hostname)
            asset["hostname"] = hostnames

        if file:
            add_helper(file, source)

        if asset:
            # create Payload
            payload = {"assets": [asset], "source": source}

            click.echo("Adding the following Data : \n{}\n".format(payload))

            # request Import Job
            data = request_data('POST', '/import/assets', payload=payload)
            click.echo("Your Import ID is : {}".format(data['asset_import_job_uuid']))
        else:
            click.echo("\nYou need to supply some information\n")
    except Error:
        click.echo("\nCheck your permissions or your API keys\n")
    except TypeError:
        click.echo("\nCheck your permissions or your API keys\n")



import click
import getpass
from .database import new_db_connection
from .dbconfig import create_keys_table, create_diff_table, create_assets_table, create_vulns_table, create_compliance_table


@click.command(help="Enter or Reset your Keys")
@click.option("-clear", is_flag=True, help="Show my Keys on the screen")
@click.option("--access_key", "--a", default="", help="Provide your Access Key")
@click.option("--secret_key", "--s", default="", help="Provide your Secret Key")
def keys(clear, access_key, secret_key):
    # create all Tables when keys are added.
    create_keys_table()
    create_diff_table()
    create_vulns_table()
    create_assets_table()
    create_compliance_table()

    if access_key == "" or secret_key == "":
        click.echo("Hey you don't have any Keys!")
        if clear:
            access_key = input("Please provide your Access Key : ")
            secret_key = input("Please provide your Secret Key : ")
        else:
            access_key = getpass.getpass("Please provide your Access Key : ")
            secret_key = getpass.getpass("Please provide your Secret Key : ")

    key_dict = (access_key, secret_key)
    database = r"navi.db"
    conn = new_db_connection(database)

    with conn:
        sql = '''INSERT or IGNORE into keys(access_key, secret_key) VALUES(?,?)'''
        cur = conn.cursor()
        cur.execute(sql, key_dict)



import click
from .agent_export import agent_export
from .database import db_query
from .tag_helper import tag_checker
from .query_export import query_export
from .agent_group_export import agent_group_export
from .user_export import user_export
from .compliance_export_csv import compliance_export_csv


@click.group(help="Export Tenable.io Data")
def export():
    pass


@export.command(help="Export All Asset data in the Navi Database to a CSV")
@click.option('--file', default="asset_data", help="Name of the file excluding 'csv'")
def assets(file):
    click.echo("\nExporting your data now. Saving {}.csv now...\n".format(file))
    asset_query = "select * from assets;"
    query_export(asset_query, file)


@export.command(help="Export All Agent data into a CSV")
def agents():
    click.echo("\nExporting your data now. Saving agent_data.csv now...\n")
    agent_export()


@export.command(help="Export Licensed Assets into a CSV")
@click.option('--file', default="licensed_data", help="Name of the file excluding 'csv'")
def licensed(file):
    click.echo("\nExporting your data now. Saving {}.csv now...\n".format(file))
    licensed_query = "SELECT ip_address, fqdn, uuid, last_licensed_scan_date from assets where last_licensed_scan_date != ' ';"
    query_export(licensed_query, file)


@export.command(help="Export All assets of a given network")
@click.argument('network_uuid')
@click.option('--file', default="network_data", help="Name of the file excluding 'csv'")
def network(network_uuid, file):
    click.echo("\nExporting your data now. Saving {}.csv now...".format(file))
    network_query = "SELECT * from assets where network=='{}';".format(network_uuid)
    query_export(network_query, file)


@export.command(help='Export assets by query to the vuln db')
@click.argument('statement')
@click.option('--file', default="query_data", help="Name of the file excluding 'csv'")
def query(statement, file):
    query_export(statement, file)


@export.command(help='Export Agents by Group name')
@click.argument('group_name')
def group(group_name):
    click.echo("\nExporting your data now.  Saving agent_group_data.csv now...")
    agent_group_export(group_name)


@export.command(help="Export All assets by tag; Include ACR and AES into a CSV")
@click.option('--c', default=None, required=True, help="Export bytag with the following Category name")
@click.option('--v', default=None, required=True, help="Export bytag with the Tag Value; requires --c and Category Name")
@click.option('--ec', default=None, help="Exclude tag from export with Tag Category; requires --ev")
@click.option('--ev', default=None, help="Exclude tag from export with Tag Value; requires --ec")
@click.option('--file', default="bytag", help="Name of the file excluding 'csv'")
@click.option('--severity', type=click.Choice(['critical', 'high', 'medium', 'low', 'info'], case_sensitive=False), multiple=True)
def bytag(c, v, ec, ev, file, severity):
    new_list = []

    tag_assets = db_query("SELECT asset_uuid from tags where tag_key='" + c + "' and tag_value='" + v + "';")

    for asset in tag_assets:
        asset_uuid = asset[0]
        # Check the tag isn't apart of the exclude tag given
        if ev:
            check_for_no = tag_checker(asset_uuid, ec, ev)
            if check_for_no == 'no':
                new_list.append(asset_uuid)
        else:
            new_list.append(asset_uuid)

    if severity:
        # If Severity is chosen then we will export vuln details
        if len(severity) == 1:
            # multiple choice values are returned as a tuple.
            # Here I break it out and put it in the format needed for sql
            new_list = tuple(new_list)
            asset_query = "select * from vulns where severity in ('{}') and asset_uuid in {};".format(severity[0], new_list)
            query_export(asset_query, file)
        else:
            new_list = tuple(new_list)
            # Here I just send the tuple in the query
            asset_query = "select * from vulns where severity in {} and asset_uuid in {};".format(severity, new_list)
            query_export(asset_query, file)
    else:
        new_list = tuple(new_list)
        asset_query = "select * from assets where uuid in {}".format(new_list)
        query_export(asset_query, file)


@export.command(help="Export User and Role information into a CSV")
def users():
    click.echo("\nExporting User Data now. Saving user-summary.csv now...\n")
    user_export()


@export.command(help="Export Compliance information into a CSV")
@click.option('--name', default=None, help="Exact name of the Audit file to be exported.  Use 'navi display audits' to "
                                           "get the right name")
@click.option('--uuid', default=None, help="UUID of the Asset for your export")
@click.option('--file', default="compliance_data.csv", help="Name of the file excluding '.csv'")
def compliance(name, uuid, file):
    click.echo("\nExporting your requested Compliance data into a CSV\n")
    compliance_export_csv(name, uuid, file)


@export.command(help="Export All Vulnerability data in the Navi Database to a CSV")
@click.option('--file', default="vuln_data", help="Name of the file excluding '.csv'")
@click.option('--severity', type=click.Choice(['critical', 'high', 'medium', 'low', 'info'], case_sensitive=False), multiple=True)
def vulns(file, severity):
    click.echo("\nExporting your data now. Saving {}.csv now...\n".format(file))

    if severity:

        if len(severity) == 1:
            # multiple choice values are returned as a tuple.
            # Here I break it out and put it in the format needed for sql
            asset_query = "select * from vulns where severity in ('{}');".format(severity[0])
        else:
            # Here I just send the tuple in the query
            asset_query = "select * from vulns where severity in {};".format(severity)
    else:
        asset_query = "select * from vulns;"

    query_export(asset_query, file)


@export.command(help="Export Vulnerabilities that have failed")
def failures():
    click.echo("\nExporting ALl vulnerabilities that failed your SLA\n This requires you run 'navi update fixed'")
    query_export("select * from fixed where pass_fail=='Fail' and state !='FIXED';", "sla_backlog")



from .ip import ip
from .find import find
from .display import display
from .delete import delete
from .api import api
from .export import export
from .lumin import lumin
from .update import update
from .add import add
from .scan import scan
from .keys import keys
from .tag import tag
from .network import network
from .user import user
from .usergroup import usergroup
from .target_group import tgroup
from .cancel import cancel
from .migrate import migrate
from .agents import agent
from .access import access
from .attribute import attribute
from .sla import sla
#from .rules import rules
from .cve_compare import compare
from .deploy import deploy


def plugin_loader(group):
    group.add_command(ip)
    group.add_command(find)
    group.add_command(display)
    group.add_command(delete)
    group.add_command(api)
    group.add_command(export)
    group.add_command(lumin)
    group.add_command(update)
    group.add_command(add)
    group.add_command(scan)
    group.add_command(keys)
    group.add_command(tag)
    group.add_command(network)
    group.add_command(user)
    group.add_command(usergroup)
    group.add_command(tgroup)
    group.add_command(cancel)
    group.add_command(migrate)
    group.add_command(agent)
    group.add_command(access)
    group.add_command(attribute)
    group.add_command(sla)
    #group.add_command(rules)
    group.add_command(compare)
    group.add_command(deploy)



import click
from .th_asset_export import asset_export
from .th_vuln_export import vuln_export
from .th_compliance_export import compliance_export
from .fixed_export import fixed_export
from .database import new_db_connection, drop_tables, create_table


def threads_check(threads):
    if threads != 10:  # Limit the amount of threads to avoid issues
        click.echo("\nUsing {} thread(s) at your request".format(threads))
        if threads not in range(1, 11):
            click.echo("Enter a value between 1 and 10")
            exit()


@click.group(help="Update the local Navi repository - saved in your current dir")
def update():
    pass


@update.command(help="Perform a full update (30d Vulns / 90d Assets); Delete the current Database")
@click.option('--threads', default=10, help="Control the threads to speed up or slow down downloads - (1-10)")
@click.option('--days', default=None, help="Limit the download to X # of days")
@click.option('--c', default=None, help="Isolate your update to a tag using the provided category")
@click.option('--v', default=None, help="Isolate your update to a tag using the provided value")
@click.option('--state', multiple=True, default=["open", "reopened"], type=click.Choice(['open', 'reopened', 'fixed']),
              help='Isolate your update to a partiular finding state')
def full(threads, days, c, v, state):

    if threads:
        threads_check(threads)

    exid = '0'

    if days is None:
        vuln_export(30, exid, threads, c, v, state)
        asset_export(90, exid, threads, c, v)
    else:
        vuln_export(days, exid, threads, c, v, state)
        asset_export(days, exid, threads, c, v)


@update.command(help="Update the Asset Table")
@click.option('--days', default='90', help="Limit the download to X # of days")
@click.option('--exid', default='0', help="Download using a specified Export ID")
@click.option('--threads', default=10, help="Control the threads to speed up or slow down downloads - (1-10)")
@click.option('--c', default=None, help="Isolate your update by a tag using the provided category")
@click.option('--v', default=None, help="Isolate your update by a tag using the provided value")
def assets(threads, days, exid, c, v):
    if threads:
        threads_check(threads)

    if exid == ' ':
        exid = '0'

    asset_export(days, exid, threads, c, v)


@update.command(help="Update the vulns Table")
@click.option('--days', default='30', help="Limit the download to X # of days")
@click.option('--exid', default='0', help="Download using a specified Export ID")
@click.option('--threads', default=10, help="Control the threads to speed up or slow down downloads - (1-10)")
@click.option('--c', default=None, help="Isolate your update by a tag using the provided category")
@click.option('--v', default=None, help="Isolate your update by a tag using the provided value")
@click.option('--state', multiple=True, default=["open", "reopened"], type=click.Choice(['open', 'reopened', 'fixed']),
              help='Isolate your update to a partiular finding state')
def vulns(threads, days, exid, c, v, state):
    if threads:
        threads_check(threads)

    if exid == ' ':
        exid = '0'

    vuln_export(days, exid, threads, c, v, list(state))


@update.command(help="Update the Compliance data")
@click.option('--days', default='30', help="Limit the download to X # of days")
@click.option('--exid', default='0', help="Download using a specified Export ID")
@click.option('--threads', default=10, help="Control the threads to speed up or slow down downloads - (1-10)")
def compliance(threads, days, exid):
    if threads:
        threads_check(threads)

    if exid == ' ':
        exid = '0'

    compliance_export(days, exid, threads)


@update.command(help="Update Navi DB with Fixed data for SLA processing")
@click.option('--c', default='', help="Tag Category name")
@click.option('--v', default='', help="Tag Value")
@click.option('--days', default='30', help="Limit the download to X # of days")
def fixed(c, v, days):
    fixed_export(c, v, days)


@update.command(help="Change the Base URL for Navi")
@click.argument('new_url')
def url(new_url):

    database = r"navi.db"
    conn = new_db_connection(database)
    drop_tables(conn, 'url')
    create_url_table = """CREATE TABLE IF NOT EXISTS url (name text, url text);"""
    create_table(conn, create_url_table)

    info = ("Custom URL", new_url)
    with conn:
        sql = '''INSERT or IGNORE into url (name, url) VALUES(?,?)'''
        cur = conn.cursor()
        cur.execute(sql, info)



import csv
import click
import datetime
import time
from csv import DictReader
from .api_wrapper import tenb_connection
import textwrap

tio = tenb_connection()


def parse_19506(plugin_output):
    """
            Parse 19506 output and return a list of data
    """
    # split the output by return
    parsed_output = plugin_output.split("\n")

    # grab the length so we can grab the seconds
    plugin_length = len(parsed_output)

    # grab the scan duration- second to the last variable
    duration = parsed_output[plugin_length - 2]

    # Split at the colon to grab the numerical value
    intial_seconds = duration.split(" : ")

    # split to remove "secs"
    number = intial_seconds[1].split(" ")

    # grab the number for our calculation
    final_number = number[0]

    # For an unknown reason, the scanner will print unknown for some assets leaving no way to calculate the time.
    if final_number != 'unknown':
        try:
            # Numerical value in seconds parsed from the plugin
            seconds = int(final_number)

            # Grab data pair and split it at the colon and grab the values
            scan_name = parsed_output[9].split(" : ")[1]
            scan_policy = parsed_output[10].split(" : ")[1]
            scanner_ip = parsed_output[11].split(" : ")[1]
            max_hosts = parsed_output[plugin_length - 8].split(" : ")[1]
            max_checks = parsed_output[plugin_length - 7].split(" : ")[1]

            # Grabbing the start time from the plugin
            start_time = parsed_output[plugin_length - 3].split(" : ")[1]
            output_list = [scan_name, scan_policy, scanner_ip, max_hosts, max_checks, start_time, seconds]

            return output_list
        except IndexError:
            return "fail"


def convert_start_date(start_time):
    # Some timezone abrivations are not parseable with strptime.
    # removing the timezone for those we can't parse
    try:
        # Set the pattern to convert into epoch
        pattern = '%Y/%m/%d %H:%M %Z'

        # Convert to Epoch
        start_time_epoch = int(time.mktime(time.strptime(start_time, pattern)))

    except ValueError:
        # timezone couldn't be used. lets remove it and calculate what we can
        pattern = '%Y/%m/%d %H:%M'

        # Split the time to remove the timezone 2-6 chars
        time_less_timezone = start_time.split(" ")

        # merge the data and time for calculation
        new_time = "{} {}".format(time_less_timezone[0], time_less_timezone[1])

        # Convert to Epoch
        start_time_epoch = int(time.mktime(time.strptime(new_time, pattern)))

    return start_time_epoch


def organize_19506_composite_data(filename):
    scanner_list = []
    total_assets_scanned_list = []
    start_scan_timestamp_list = []
    timestamp_plus_duration_list = []

    with open(filename) as fobj:

        for row in DictReader(fobj):
            plugin_output = row['Plugin Output']
            asset_uuid = row['Asset UUID']

            # Parse the plugin and accept the list
            plugin_list = parse_19506(plugin_output)

            if plugin_list != 'fail':
                seconds = plugin_list[6]

                scan_name = plugin_list[0]
                scan_policy = plugin_list[1]

                # There could be more than one scanner; create a list to store them all
                if plugin_list[2] not in scanner_list:
                    scanner_list.append(plugin_list[2])

                # Grabbing the start time from the plugin
                start_time_epoch = convert_start_date(plugin_list[5])

                # Add to list to calculate the first asset scanned
                start_scan_timestamp_list.append(start_time_epoch)

                # Add epoch and seconds to get end time.  This will be used to find the last asset scanned
                timestamp_plus_duration_list.append(start_time_epoch + seconds)

                # All assets and 19506 seconds in a tuple
                total_assets_scanned_list.append((asset_uuid, seconds))

                # calculate the total for AVG calc
            total = 0
            for secs in total_assets_scanned_list:
                total += secs[1]

    organize_composite_data = {"Scanner List": scanner_list, "Total Assets": total_assets_scanned_list,
                               "Start Scan List": start_scan_timestamp_list,
                               "Duration List": timestamp_plus_duration_list,
                               "Total": total,
                               "Scan Name": scan_name,
                               "Scan Policy": scan_policy}

    return organize_composite_data


def decorate_19506_data(filename):
    # Grab the composite data
    composite_data = organize_19506_composite_data(filename)

    # reassign the composite data for calculations
    total_assets_scanned_list = composite_data['Total Assets']

    scanner_list = composite_data["Scanner List"]

    start_scan_timestamp_list = composite_data["Start Scan List"]

    timestamp_plus_duration_list = composite_data["Duration List"]

    total = composite_data["Total"]

    scan_policy = composite_data["Scan Policy"]

    scan_name = composite_data["Scan Name"]

    try:

        # Avg scan time per asset
        asset_average_scantime = total / len(total_assets_scanned_list)

        # total Scan time is: the oldest (scan_date in 19506 + scan duration in 19506) minus the earliest scan date in 19506
        total_calculated_scan_duration = (max(timestamp_plus_duration_list) - min(start_scan_timestamp_list))

        # The asset that took the longest will be the biggest number
        longest_scanned = max(total_assets_scanned_list, key=lambda uuid: uuid[1])

        # The asset that took the shortest will be the smallest number
        shortest_scanned = min(total_assets_scanned_list, key=lambda uuid: uuid[1])

        # Instead of providing the UUID, provide the URL that points to the Asset details page
        short_asset_special_url = "https://cloud.tenable.com/tio/app.html#/assets-uw/hosts-assets/details/{}/findings?uw_asset_details_findings_nessus.st=severity.1".format(
            shortest_scanned[0])
        long_asset_special_url = "https://cloud.tenable.com/tio/app.html#/assets-uw/hosts-assets/details/{}/findings?uw_asset_details_findings_nessus.st=severity.1".format(
            longest_scanned[0])

        # collect the data in a list and return it
        trend_data = [str(time.strftime('%Y-%m-%d %H:%M:%S', time.localtime(min(start_scan_timestamp_list)))),
                      str(time.strftime('%Y-%m-%d %H:%M:%S', time.localtime(max(timestamp_plus_duration_list)))),
                      scan_policy, scanner_list, scan_name, len(total_assets_scanned_list),
                      str(datetime.timedelta(seconds=asset_average_scantime)),
                      str(datetime.timedelta(seconds=total_calculated_scan_duration)),
                      str(datetime.timedelta(seconds=int(shortest_scanned[1]))), short_asset_special_url,
                      str(datetime.timedelta(seconds=int(longest_scanned[1]))), long_asset_special_url]
        return trend_data, total_calculated_scan_duration

    except ZeroDivisionError:
        click.echo("\nScan history had No data.\n")


def download_csv_by_plugin_id(scan_id, hist_id):
    filename = f'{scan_id}-{hist_id}-report.csv'

    # Stream the report to disk
    with open(filename, 'wb') as fobj:
        tio.scans.export(scan_id, ('plugin.id', 'eq', '19506'),
                         format='csv', fobj=fobj, history_id=hist_id)

    return filename


def trend_by_scan_id(scanid):

    with open('Trending_report_scan_{}.csv'.format(scanid), mode='w', encoding='utf-8', newline="") as trend_file:
        trend_writer = csv.writer(trend_file, delimiter=',', quotechar='"')
        header = ["Scan URl", "Scan Start", "Last Asset Scanned", "Scan Policy", "Scanner IP", "Scan Name",
                  "Total Assets",
                  "Average Scan Duration", "Reported Time", "Indexing Time", "Scan Duration", "Shortest Asset time",
                  "Shortest Scanned Asset", "Longest Asset time", "longest Scanned Asset"]
        trend_writer.writerow(header)

        for hist in tio.scans.history(scanid):  # scan_hist['history']:
            if not hist['is_archived']:
                if hist['status'] == 'completed':
                    # Lets get the Reported Scan Duration
                    reported_scan_start = hist['time_start']
                    reported_scan_end = hist['time_end']
                    total_reported_scan_duration = reported_scan_end - reported_scan_start

                    # Simple Delta from Tenable.io reported Scan times
                    reported_scan_duration = str(datetime.timedelta(seconds=total_reported_scan_duration))

                    click.echo("\nDownloading scan {}, history {} details into a csv called {}-{}.csv for "
                               "parsing and manual auditing\n".format(scanid, hist['id'], scanid, hist['id']))

                    # Download the Scan data
                    scan_file_name = download_csv_by_plugin_id(scanid, hist['id'])

                    # Parse the downloaded file
                    scan_details, total_calculated_scan = decorate_19506_data(scan_file_name)

                    # Processing time therefore is the reported duration - the total scan duration
                    processing = total_reported_scan_duration - total_calculated_scan

                    # Constructing the scan URL for ease of use
                    scan_url = "https://cloud.tenable.com/tio/app.html#/assess/scans/vm-scans/folders/1/scan-details/{}/{}/" \
                               "by-plugin/vulnerability-details/19506/details".format(scanid, hist['scan_uuid'])

                    # Insert the reported duration and scan url
                    scan_details.insert(0, scan_url)
                    scan_details.insert(8, reported_scan_duration)
                    scan_details.insert(9, datetime.timedelta(seconds=processing))

                    # Write to File
                    trend_writer.writerow(scan_details)


def display_data(scanid):

    with open('Trending_report_scan_{}.csv'.format(scanid), mode='r', encoding='utf-8', newline="") as trend_file:
        click.echo()
        click.echo("*" * 100)
        click.echo("\nThis data is derived from the 19506 plugins found in the last 30 days of available scan history of scan {}"
                   "\nThe start time stamps and the durations were used to calculate the total scan times"
                   "\nThe file Trending_report_scan_{}.csv has all of the available data, including links to assets and scan history\n".format(scanid, scanid))
        click.echo("*" * 100)
        click.echo("\n{:20} {:25} {:10} {:20} {:20} {:20} {:20} {:25}".format("Scan Start", "Scanner IP(s)", "Assets", "Reported",
                                                                              "Indexing", "Duration", "Average",
                                                                              "Longest Asset time"))
        click.echo("-" * 150)

        for detail in DictReader(trend_file):
            click.echo("{:20} {:25} {:10} {:20} {:20} {:20} {:20} {:25}".format(str(detail["Scan Start"]), textwrap.shorten(str(detail["Scanner IP"]), width=25),
                                                                                str(detail["Total Assets"]), str(detail["Reported Time"]),
                                                                                str(detail["Indexing Time"]), str(detail["Scan Duration"]),
                                                                                detail["Average Scan Duration"], str(detail["Longest Asset time"])))

    click.echo()






import time
import threading
import click
from queue import Queue
from sqlite3 import Error
from .api_wrapper import request_data
from .database import new_db_connection, drop_tables, insert_compliance
from .dbconfig import create_compliance_table

lock = threading.Lock()

q = Queue()


def worker():
    # The worker thread pulls an item from the queue and processes it
    while True:
        item = q.get()
        chunk_num = item[63:]
        # parse_data(request_data('GET', item))
        data = request_data('GET', item)
        parse_data(data, chunk_num)
        q.task_done()


def parse_data(chunk_data, chunk_number):
    database = r"navi.db"
    comply_conn = new_db_connection(database)
    comply_conn.execute('pragma journal_mode=wal;')
    comply_conn.execute('pragma cashe_size=-10000')
    comply_conn.execute('pragma synchronous=OFF')
    with comply_conn:
        try:
            # loop through all of the compliance info in this chunk
            for finding in chunk_data:
                # create a blank list to append asset details
                finding_list = []
                # Try block to ignore assets without IPs
                try:
                    try:
                        asset_uuid = finding['asset_uuid']
                        finding_list.append(asset_uuid)
                    except KeyError:
                        finding_list.append(" ")

                    try:
                        actual_value = finding['actual_value']
                        finding_list.append(actual_value)
                    except KeyError:
                        finding_list.append(" ")

                    try:
                        audit_file = finding['audit_file']
                        finding_list.append(audit_file)
                    except KeyError:
                        finding_list.append(" ")

                    try:
                        check_id = finding['check_id']
                        finding_list.append(check_id)
                    except KeyError:
                        finding_list.append(" ")

                    try:
                        check_info = finding['check_info']
                        finding_list.append(check_info)
                    except KeyError:
                        finding_list.append(" ")

                    try:
                        check_name = finding['check_name']
                        finding_list.append(check_name)
                    except KeyError:
                        finding_list.append(" ")

                    try:
                        expected_value = finding['expected_value']
                        finding_list.append(expected_value)
                    except KeyError:
                        finding_list.append(" ")

                    try:
                        first_seen = finding['first_seen']
                        finding_list.append(first_seen)
                    except KeyError:
                        finding_list.append(" ")

                    try:
                        last_seen = finding['last_seen']
                        finding_list.append(last_seen)
                    except KeyError:
                        finding_list.append(" ")

                    try:
                        plugin_id = finding['plugin_id']
                        finding_list.append(plugin_id)
                    except KeyError:
                        finding_list.append(" ")

                    # Need to break this into it's own table.
                    try:
                        reference = str(finding['reference'])
                        finding_list.append(reference)
                    except KeyError:
                        finding_list.append(" ")

                    try:
                        see_also = finding['see_also']
                        finding_list.append(see_also)
                    except KeyError:
                        finding_list.append(" ")

                    try:
                        solution = finding['solution']
                        finding_list.append(solution)
                    except KeyError:
                        finding_list.append(" ")

                    try:
                        check_status = finding['status']
                        finding_list.append(check_status)
                    except KeyError:
                        finding_list.append(" ")

                    try:
                        insert_compliance(comply_conn, finding_list)
                    except Error as e:
                        click.echo(e)

                except IndexError:
                    click.echo("skipped one")
        except TypeError:
            click.echo("Your Export has no data.  It may have expired")
            click.echo("Error on Chunk {}".format(chunk_number))

    click.echo("Chunk {} Finished".format(chunk_number))
    comply_conn.close()


def compliance_export(days, ex_uuid, threads):
    start = time.time()

    database = r"navi.db"
    drop_conn = new_db_connection(database)
    drop_conn.execute('pragma journal_mode=wal;')

    # Right now we just drop the table.  Eventually I will actually update the database
    drop_tables(drop_conn, 'compliance')

    create_compliance_table()

    # Set URLS for threading
    urls = []

    # Set the payload to the maximum number of assets to be pulled at once
    day = 86400
    new_limit = day * int(days)
    day_limit = time.time() - new_limit
    pay_load = {"num_findings": 50, "filters": {'last_seen': int(day_limit)}}
    try:

        if ex_uuid == '0':
            # request an export of the data
            export = request_data('POST', '/compliance/export', payload=pay_load)

            # grab the export UUID
            ex_uuid = export['export_uuid']
            click.echo('\nRequesting Compliance Export with ID : {}'.format(ex_uuid))

            # set a variable to True for our While loop
            not_ready = True
        else:
            click.echo("\nUsing your Export UUID\n")
            not_ready = True

        # now check the status
        status = request_data('GET', '/compliance/export/' + ex_uuid + '/status')

        # status
        click.echo("Status : {}".format(str(status["status"])))

        # loop to check status until finished
        while not_ready is True:
            # Pull the status, then pause 5 seconds and ask again.
            if status['status'] == 'PROCESSING' or 'QUEUED':
                time.sleep(2.5)
                status = request_data('GET', '/compliance/export/' + ex_uuid + '/status')
                # click.echo("Status : " + str(status["status"]))

            # Exit Loop once confirmed finished
            if status['status'] == 'FINISHED':
                ptime = time.time()
                click.echo("\nProcessing Time took : {}".format(str(ptime - start)))

                # Display how many chunks there are
                avail = len(status['chunks_available'])
                click.echo("\nChunks Available - {}".format(avail))
                click.echo("Downloading chunks now...hold tight...This can take some time\n")
                not_ready = False

            # Tell the user an error occurred
            if status['status'] == 'ERROR':
                click.echo("\nT.io Error occurred\n Try again!")
                exit()

        # grab all of the chunks and craft the URLS for threading
        for y in status['chunks_available']:
            urls.append('/compliance/export/' + ex_uuid + '/chunks/' + str(y))

        for i in range(threads):
            t = threading.Thread(target=worker)
            t.daemon = True  # thread dies when main thread (only non-daemon thread) exits.
            t.start()

        for item in range(len(urls)):
            q.put(urls[item])

        q.join()
        end = time.time()
        click.echo("Compliance Update Time took : {}\n".format(str(end - start)))

    except KeyError:
        click.echo("Well this is a bummer; you don't have permissions to download Asset data :( ")

    except TypeError:
        click.echo("You may not be authorized or your keys are invalid")



import click
import os


@click.command(help="Deploy Navi Tag Center using a Docker container: navigate to http://localhost:5000")
def deploy():
    if click.confirm('This command downloads the silentninja/navi-ent docker container and runs it on port 5000 using the current navi database. Deploy?'):
        try:
            os.system("docker run -d -p 5000:5000 --mount type=bind,source=\"$(pwd)\",target=/usr/src/app/data silentninja/navi-ent")
        except os.error:
            click.echo("You might not have Docker installed")



import click
from .api_wrapper import request_no_response, request_data
from .user import get_user_id


def create_group(group_name):
    payload = {'name': group_name}
    # Using the Delete request because of an API Return issue.
    data = request_no_response("POST", "/groups", payload=payload)

    return data


def add_users(user_id, group_id):
    url = "/groups/{}/users/{}".format(group_id, user_id)
    # Using the Delete request because of an API Return issue.
    request_no_response("POST", url)


def remove_user(user_id, group_id):
    url = "/groups/{}/users/{}".format(group_id, user_id)
    # Using the Delete request because of an API Return issue.
    request_no_response("DELETE", url)


def get_group_id(group_name):
    data = request_data("GET", "/groups")
    group_id = 0
    group_uuid = 0
    for group in data["groups"]:

        if group_name == group["name"]:
            group_id = group["id"]
            group_uuid = group['uuid']
    return group_id, group_uuid


@click.group(help="Create a user group or Add/remove a user from a user group")
def usergroup():
    pass


@usergroup.command(help="Create a new user group")
@click.option("--name", default='', required=True, help="The Name of the user group")
def create(name):
    # Check to see if the group already exists
    group_id, group_uuid = get_group_id(name)
    if group_id == 0:
        create_group(name)
    else:
        print("Your Group already exists. Here is the group id {}".format(group_id))


@usergroup.command(help="Add a User to a user group")
@click.option("--name", default='', required=True, help="The Name of the user group")
@click.option("--user", default='', required=True, help="The User Name to be added")
def add(name, user):
    user_id, user_uuid = get_user_id(user)
    group_id, group_uuid = get_group_id(name)
    add_users(user_id, group_id)


@usergroup.command(help="Remove a User from a user group")
@click.option("--name", default='', required=True, help="The Name of the group")
@click.option("--user", default='', required=True, help="The User Name to be removed")
def remove(name, user):
    user_id, user_uuid = get_user_id(user)
    group_id, group_uuid = get_group_id(name)
    remove_user(user_id, group_id)



import click
from .fixed_export import calculate_sla, reset_sla, print_sla


@click.group(help="Set, Reset and Calculate SLAs")
def sla():
    pass


@sla.command(help="Overwrite your SLA information")
@click.option("--critical", default='', help="Set your Critical Vulnerability SLA")
@click.option("--high", default='', help="Set your High Vulnerability SLA")
@click.option("--medium", default='', help="Set your Meduim SLA")
@click.option("--low", default='', help="Set your Low SLA")
def reset(critical, high, medium, low):

    if critical == '' and high == '' and medium == '' and low == '':
        print("You Entered Nothing, but choose to reset your SLA.  I'm using the Defaults")
        # Set Defaults: user could only select one
        reset_sla(7, 14, 30, 180)
        print_sla()
    else:
        if critical == '':
            critical = 7

        if high == '':
            high = 14

        if medium == '':
            medium = 30

        if low == '':
            low = 180

        reset_sla(critical, high, medium, low)
        print_sla()


@sla.command(help="Calculate SLA times")
def calculate():
    try:
        calculate_sla("total")
        calculate_sla("critical")
        calculate_sla("high")
        calculate_sla("medium")
        calculate_sla("low")
        click.echo()
    except:
        click.echo("\n You need to run `navi update fixed` first\n")



import click
import textwrap
import time
import datetime
from .display import get_scanners
from .api_wrapper import request_data, tenb_connection
from .error_msg import error_msg
from .scan_evaluation import evaluate_a_scan
import os
from .scan_efficentcy import display_data, trend_by_scan_id

tio = tenb_connection()


def get_scans_by_owner(owner):
    data = request_data('GET', '/scans')
    scan_data = []

    for get_scan in data['scans']:
        scan_id = get_scan['id']
        scan_owner = get_scan['owner']
        scan_name = get_scan['name']
        scan_uuid = get_scan['wizard_uuid']
        if scan_owner == owner:
            scan_data.append((scan_name, scan_id, owner, scan_uuid))
    return scan_data


def get_scans():
    scan_list = []
    for move_scan in tio.scans.list():
        if move_scan['status'] == 'completed':
            scan_list.append(move_scan['id'])
    return scan_list


def get_owner_uuid(owner):
    users = request_data('GET', '/users')
    user_uuid = 0

    for user in users['users']:
        if user['username'] == owner:
            user_uuid = user['id']
    return user_uuid


def get_targets(scan_id):
    get_scan_details = request_data('GET', '/scans/' + str(scan_id))
    try:
        targets = get_scan_details['info']['targets']
    except TypeError:
        targets = ""
    except KeyError:
        targets = ""

    try:
        tag_targets = get_scan_details['info']['tag_targets']
    except TypeError:
        tag_targets = ""

    scanner_name = get_scan_details['info']['scanner_name']

    return targets, tag_targets, scanner_name


def get_scanner_id(scan_name):
    scanners = request_data('GET', '/scanners')
    for scanner in scanners['scanners']:
        if scanner['name'] == scan_name:
            return scanner['id']


def scan_details(scan_id):
    try:
        data = request_data('GET', '/scans/' + str(scan_id))
        try:
            click.echo("\nScan Details for Scan ID : {}\n".format(scan_id))
            click.echo("Notes: \b")
            try:
                click.echo(data['notes'][0]['message'])
            except IndexError:
                pass
            click.echo("\nVulnerability Counts")
            click.echo("-" * 20)
            click.echo("Critical : {}".format(data['hosts'][0]['critical']))
            click.echo("high : {}".format(data['hosts'][0]['high']))
            click.echo("medium : {}".format(data['hosts'][0]['medium']))
            click.echo("low : {}".format(data['hosts'][0]['low']))
            try:
                click.echo("-" * 15)
                click.echo("Score : {}".format(data['hosts'][0]['score']))
            except IndexError:
                pass
            click.echo("\n{:10s} {:128s} {}".format("Plugin", "Vulnerability Details", "Vuln Count"))
            click.echo("-" * 150)

            for vulns in data['vulnerabilities']:
                if vulns['severity'] != 0:
                    click.echo("{:10s} {:128s} {}".format(str(vulns['plugin_id']), textwrap.shorten(str(vulns['plugin_name']), 128), vulns['count']))
            click.echo()
        except TypeError:
            click.echo("Check the scan ID")
    except Exception as E:
        click.echo("Check the scan ID\n")
        click.echo("Error: \n {}".format(E))
        exit()


def scan_hosts(scan_id):
    try:
        data = request_data('GET', '/scans/' + str(scan_id))
        host_list = []
        try:
            click.echo("\nHosts Found by Scan ID : {}\n".format(scan_id))
            click.echo("{:20s} {:45s} {:10} {:10s} {:10s} {:10s} {:10s} ".format("IP Address", "UUID", "Score", "Critical", "High", "Medium", "Low"))
            click.echo("-"*150)
            for host in data['hosts']:
                host_list.append(host)
                click.echo("{:20s} {:45s} {:10} {:10s} {:10s} {:10s} {:10s}".format(host['hostname'], str(host['uuid']), str(host['score']), str(host['critical']), str(host['high']), str(host['medium']), str(host['low'])))

            click.echo()
            click.echo("\nTotal Assets Scanned: {}\n".format(len(host_list)))
        except KeyError:
            click.echo("There was an Error.  It could be the scan was Aborted, canceled or Archived.\n")
            click.echo("Status: {}".format(data['info']['status']))
            click.echo("Archived? {}".format(data['info']['is_archived']))

        except TypeError:
            click.echo("Check the scan ID")
    except Exception as E:
        error_msg(E)


@click.group(help="Create and Control Scans")
def scan():
    pass


@scan.command(help="Quickly Scan a Target")
@click.argument('targets')
@click.option('--plugin', default='', help="Plugin required for Remediation Scan")
@click.option('--cred', default='', help="Scan using a UUID of your intended credentials")
@click.option('-discovery', is_flag=True, help="Scan using the Discovery Template")
@click.option('--custom', default='', help="Scan using a custom Scan Template")
@click.option('--scanner', default='', help="Scan using a specific Scanner ID")
@click.option('--policy', default='', help="Scan using a custom Policy by Policy ID")
def create(targets, plugin, cred, discovery, custom, scanner, policy):
    # If a Template isn't chosen we will assume a Basic Network scan
    template = '731a8e52-3ea6-a291-ec0a-d2ff0619c19d7bd788d6be818b65'

    if discovery:
        template = 'bbd4f805-3966-d464-b2d1-0079eb89d69708c3a05ec2812bcf'

    if len(custom) == 52:
        template = custom

    if scanner:
        scanner_id = scanner
    else:
        click.echo("Here are the available scanners")
        click.echo("Remember, don't pick a Cloud scanner for an internal IP address")
        click.echo("Remember also, don't chose a Webapp scanner for an IP address")
        get_scanners()
        scanner_id = input("What scanner do you want to scan with ?.... ")

    click.echo("creating your scan of : {}  Now...".format(targets))

    # Begin Payload Creation
    payload = dict(uuid=template,
                   settings={"name": "navi Created Scan of " + targets,
                             "enabled": "True",
                             "scanner_id": scanner_id,
                             "text_targets": targets})

    if cred:
        cred_data = request_data('GET', '/credentials/' + cred)
        try:
            cred_cat_name = cred_data['category']['name']
            cred_type_name = cred_data['type']['name']

            # Add credentials to payload
            payload["credentials"] = {"add": {cred_cat_name: {cred_type_name: [{"id": cred}]}}}

        except KeyError:
            click.echo("\nCheck your Credential UUID\n")
            exit()

    if policy:
        custom_template = 'ab4bacd2-05f6-425c-9d79-3ba3940ad1c24e51e1f403febe40'
        try:
            # Change template
            payload["uuid"] = custom_template

            # Add policy to payload
            payload["settings"]["policy_id"] = policy

        except KeyError:
            click.echo("\nCheck your Credential UUID\n")
            exit()

    if plugin != '':
        advanced_template = 'ad629e16-03b6-8c1d-cef6-ef8c9dd3c658d24bd260ef5f9e66'

        # Change template
        payload["uuid"] = advanced_template

        # Add plugins to dictionary
        payload["enabled_plugins"] = [plugin]
    print(payload)
    # create a new scan
    data = request_data('POST', '/scans', payload=payload)

    # pull scan ID after Creation
    scan_id = str(data["scan"]["id"])

    # launch Scan
    request_data('POST', '/scans/' + scan_id + '/launch')

    click.echo("I started your scan, your scan ID is: {}".format(scan_id))


@scan.command(help="Start a valid Scan by Scan ID")
@click.argument('scan_id')
def start(scan_id):
    tio.scans.launch(scan_id)


@scan.command(help="Get Scan Status by Scan ID")
@click.argument('Scan_id')
def status(scan_id):
    click.echo("\nLast Status update : {}\n".format(tio.scans.status(scan_id)))


@scan.command(help="Resume a paused Scan by Scan ID")
@click.argument('scan_id')
def resume(scan_id):
    tio.scans.resume(scan_id)


@scan.command(help="Pause a running Scan by Scan ID")
@click.argument('Scan_id')
def pause(scan_id):
    tio.scans.pause(scan_id)


@scan.command(help="Stop a Running Scan by Scan ID")
@click.argument('scan_id')
def stop(scan_id):
    tio.scans.stop(scan_id)


@scan.command(help="Change scan ownership using user logins")
@click.option('--owner', default='', help='Current Owner login.')
@click.option('--new', default='', help='New Owner login.')
@click.option('--who', default='', help="Check what scans a user owns")
@click.option('-v', is_flag=True, help="Verbose output")
def change(owner, new, who, v):

    if who:
        # This should be moved to display commands.
        who_scans = get_scans_by_owner(who)

        click.echo("\n{:80s} {:10} {}".format("Scan Name", "Scan ID", "Scan Owner"))
        click.echo("-" * 150)
        click.echo()

        for who_scan in who_scans:
            click.echo("{:80s} {:10} {}".format(str(who_scan[0]), str(who_scan[1]), str(who_scan[2])))
        click.echo()

    if owner:
        owner_scans = get_scans_by_owner(owner)
        new_owner_uuid = get_owner_uuid(new)

        click.echo("\n*** Scans that have not run, will produce HTTP 400 errors ***")
        click.echo("\nYou Scans are being converted now.")
        click.echo("\nThis can take some time")

        for owner_scan in owner_scans:
            scan_uuid = owner_scan[3]
            scan_name = owner_scan[0]
            scan_id = owner_scan[1]
            targets, tag_targets, scanner_name = get_targets(scan_id)
            payload = dict(uuid=scan_uuid, settings={"name": scan_name,
                                                     "owner_id": new_owner_uuid,
                                                     "tag_targets": tag_targets,
                                                     "text_targets": targets,
                                                     "scanner_id": get_scanner_id(scanner_name)
                                                     })

            request_data('PUT', '/scans/' + str(scan_id), payload=payload)
            if v:
                click.echo("\nDone - Payload: {}".format(str(payload)))


@scan.command(help="Display Scan Details")
@click.argument('scan_id')
def details(scan_id):
    scan_details(scan_id)


@scan.command(help="Display Hosts Found by a Scan ID")
@click.argument('scan_id')
def hosts(scan_id):
    scan_hosts(scan_id)


@scan.command(help="Display History index for a Scan ID that is NOT archived(older than 35 days)")
@click.argument('scan_id')
def history(scan_id):
    try:
        total = 0
        data_list = []
        data = request_data("GET", "/scans/{}/history".format(scan_id))
        click.echo("\nHistory IDs and Status for scan ID {}".format(scan_id))
        click.echo("-" * 40)
        click.echo("\n{:15s} {:15s} {:20s} {:20s} {}".format("History ID", "Scan Status", "Start time", "End Time", "Duration (H:M:S)"))
        click.echo("-" * 100)
        for hist in data['history']:
            if not hist['is_archived']:
                duration = hist['time_end'] - hist['time_start']
                data_list.append(duration)
                total += duration
                click.echo("{:15s} {:15s} {:20} {:20} {}".format(str(hist['id']), hist['status'],
                                                                 str(datetime.datetime.fromtimestamp(hist['time_start'])),
                                                                 str(datetime.datetime.fromtimestamp(hist['time_end'])),
                                                                 str(datetime.timedelta(seconds=duration))))
        click.echo()
        click.echo("\nAverage Scan time: {}\n\n".format(datetime.timedelta(seconds=(total / len(data_list)))))
    except KeyError:
        click.echo("\nTry another Scan ID; This one isn't working")
    except ZeroDivisionError:
        click.echo("\nTry another Scan ID; This one is archived, imported or was canceled")


@scan.command(help="Display the Latest scan information")
def latest():
    try:
        data = request_data('GET', '/scans')
        time_list = []
        e = {}
        for x in data["scans"]:
            # keep UUID and Time together
            # get last modication date for duration computation
            epoch_time = x["last_modification_date"]
            # get the scanner ID to display the name of the scanner
            d = x["id"]
            # need to identify type to compare against pvs and agent scans
            scan_type = str(x["type"])
            # don't capture the PVS or Agent data in latest
            while scan_type not in ['pvs', 'agent', 'webapp', 'lce']:
                # put scans in a list to find the latest
                time_list.append(epoch_time)
                # put the time and id into a dictionary
                e[epoch_time] = d
                break

        # find the latest time
        grab_time = max(time_list)

        # get the scan with the corresponding ID
        grab_uuid = e[grab_time]

        # turn epoch time into something readable
        epock_latest = time.strftime("%a, %d %b %Y %H:%M:%S +0000", time.localtime(grab_time))
        click.echo("\nThe last Scan run was at {}".format(epock_latest))
        scan_details(str(grab_uuid))
    except Exception as E:
        error_msg(E)


@scan.command(help="Move Scans from one T.io container to another")
@click.option('--a', default=None, help="T.io Destination Access Key")
@click.option('--s', default=None, help="T.io Destination Secret Key")
@click.option('--limit', default=1, help="Limit History download to a number of completed scans; default is 1")
@click.option('--scanid', default=None, help="Limit the Download to one scan ID")
def move(a, s, limit, scanid):
    from tenable.io import TenableIO
    # Source Container Keys
    src = tenb_connection()

    # Destination Container Keys
    dst = TenableIO(access_key=a, secret_key=s)

    def get_all_scans():
        move_scan_list = []
        for move_scan in src.scans.list():
            if move_scan['status'] == 'completed':
                move_scan_list.append(move_scan['id'])
        return move_scan_list

    def get_history(move_scan_id):
        move_limit = 0
        move_history_list = []

        for scan_instance in src.scans.history(scan_id=move_scan_id):

            # Change the limit to how many scan histories you want to pull.
            if move_limit == limit:
                break

            if scan_instance['status'] == "completed":
                move_limit = move_limit + 1
                move_history_list.append(scan_instance['id'])
        return move_history_list

    def scan_mover(scan_mover_list):
        for history_ids in scan_mover_list:

            # pull the last X(limit) scan jobs
            history_list = get_history(history_ids)
            click.echo("Here is the history for {}:\n {}\n".format(history_ids, history_list))

            # iterate over each scan history
            for move_scans in history_list:
                scan_name = "{}_{}".format(history_ids, move_scans)

                click.echo("Exporting Scan ID:{}, with history_id: {} now\n".format(history_ids, move_scans))

                # export the scan
                with open('{}.nessus'.format(str(scan_name)), 'wb') as nessus:
                    src.scans.export(history_ids, fobj=nessus)

                nessus.close()

                click.echo("Importing Scan ID: {}, with history_id: {} now\n".format(history_ids, move_scans))

                # import the scan
                with open('{}.nessus'.format(str(scan_name)), 'rb') as file:
                    dst.scans.import_scan(fobj=file)
                file.close()

                # delete the scan
                os.remove('{}.nessus'.format(str(scan_name)))

    if scanid:
        single_scan = [scanid]
        scan_mover(single_scan)
    else:
        # Grab all of the 'remote' scans
        scan_list = get_all_scans()
        click.echo("Grabbing the list {}\n".format(scan_list))

        scan_mover(scan_list)


@scan.command(help="Export and Import scans between T.io and T.sc")
@click.option('--un', default=None, help="T.sc User Name")
@click.option('--pw', default=None, help="T.sc password")
@click.option('--a', default=None, help="T.sc Destination Access Key")
@click.option('--s', default=None, help="T.sc Destination Secret Key")
@click.option('--host', default=None, help="T.sc IP Address")
@click.option('-io', is_flag=True, help="Use T.io as the Host to send T.sc scans to T.io")
@click.option('--scanid', default=None, help="Limit the Download to one scan ID")
@click.option('--repoid', default=None, help="T.sc Repository to import the scan data into")
@click.option('-allscans', is_flag=True, help="Move All remote and completed scans to a single repository")
def bridge(un, pw, host, scanid, repoid, a, s, allscans, io):
    from tenable.sc import TenableSC

    sc = TenableSC(host)

    if un and pw:
        sc.login(un, pw)

    if a and s:
        sc.login(access_key=a, secret_key=s)
    # Turn into a function, create an if statement with a for loop, looping over every
    # completed scan.

    def download_import_scan(sid, repo_id):
        try:
            click.echo("\nExporting your Scan ID: {} now\n".format(sid))

            with open('{}.nessus'.format(str(sid)), 'wb') as nessus:
                tio.scans.export(sid, fobj=nessus)

            click.echo("Importing your Scan into Repo {} at https://{}\n".format(repo_id, host))

            with open('{}.nessus'.format(str(sid))) as file:
                sc.scan_instances.import_scan(file, repo_id)

            # delete the scan
            os.remove('{}.nessus'.format(str(sid)))
        except:
            pass

    if io:
        click.echo("\nThis is going to download all of your Tenable.sc scans and import them into Tenable.io.")
        click.echo("To cancel this process after it has started, close your terminal\n")
        if click.confirm('Confirm you want to proceed'):

            def export_import_sc_scans(sc_scan_id):

                try:
                    click.echo("\nExporting your Scan ID: {} now\n".format(sc_scan_id))

                    with open('{}.nessus'.format(str(sc_scan_id)), 'wb') as nessus:
                        sc.scan_instances.export_scan(sc_scan_id, nessus)
                    nessus.close()

                    click.echo("Importing your Scan to Tenable.io\n".format())

                    with open('{}.nessus'.format(str(sc_scan_id)), 'rb') as file:
                        tio.scans.import_scan(fobj=file)
                    file.close()

                    # delete the scan
                    os.remove('{}.nessus'.format(str(sc_scan_id)))
                except:
                    pass
            if allscans:
                for sc_scan in sc.scan_instances.list()['usable']:
                    if sc_scan['status'] == 'Completed':
                        export_import_sc_scans(sc_scan['id'])
            else:
                export_import_sc_scans(scanid)

    else:
        if allscans:
            # Put all scans into a list
            scan_ids = get_scans()

            for scan_id in scan_ids:
                download_import_scan(scan_id, repoid)
        else:
            download_import_scan(scanid, repoid)

    sc.logout()


@scan.command(help="Evaluate Scan times")
@click.option("--scanid", default="", help="A Scan ID you want to evaluate")
@click.option("--histid", default="", help="A Specific History ID")
@click.option("-full", is_flag=True, help="Evaluate entire available history")
def evaluate(scanid, histid, full):
    if full:
        try:
            display_data(scanid)

        except FileNotFoundError:
            if click.confirm("\nThis command will download each scan history filtered by only the 19506 plugin,"
                             "then parse each output while calculating important statistics.\n It can take from 5 mins "
                             "to over an hour depending on scan frequency and asset count\n"):
                trend_by_scan_id(scanid)
                display_data(scanid)

        except FileExistsError:
            if click.confirm("\nThis command will download each scan history filtered by only the 19506 plugin,"
                             "then parse each output while calculating important statistics.\n It can take from 5 mins "
                             "to over an hour depending on scan frequency and asset count\n"):
                trend_by_scan_id(scanid)
                display_data(scanid)
    else:
        evaluate_a_scan(scanid, histid)



import click
from .database import db_query
from .tag_helper import tag_checker
from .api_wrapper import request_data


@click.command(help="Adjust ACRs in Lumin by tag")
@click.option('--acr', default='', help='Set the ACR')
@click.option('--c', default=None, required=True,  help="Tag Category to use")
@click.option('--v', default=None, required=True, help="Tag Value to use")
@click.option('--note', default="navi Generated", help="Enter a Note to your ACR update")
@click.option('-business', '-b', is_flag=True, help="Add Business Critical To ACR Change Reason(s)")
@click.option('-compliance', '-c', is_flag=True, help="Add Compliance To ACR Change Reason(s)")
@click.option('-mitigation', '-m', is_flag=True, help="Add Mitigation Controls To ACR Change Reason(s)")
@click.option('-development', '-d', is_flag=True, help="Add Development To ACR Change Reason(s)")
def lumin(acr, v, c, note, business,  compliance, mitigation, development):
    choice = []

    if business:
        choice.append("Business Critical")

    if compliance:
        choice.append("In Scope For Compliance")

    if mitigation:
        choice.append("Existing Mitigation Control")

    if development:
        choice.append("Dev Only")

    if not business and not mitigation and not compliance and not development:
        choice.append("Key Drivers does not match")

    if note != 'navi Generated':
        choice.append("Other")

    if int(acr) in range(1, 11):

        data = db_query("SELECT asset_uuid from tags where tag_key='" + c + "' and tag_value='" + v + "';")

        lumin_list = []

        for asset in data:
            uuid = asset[0]

            # First make sure the asset doesn't have a NO:UPDATE tag
            check_for_no = tag_checker(uuid, "NO", "UPDATE")
            if check_for_no == 'no':
                lumin_list.append(uuid)

        for asset_uuid in lumin_list:
            note = note + " - navi"
            lumin_payload = [{"acr_score": int(acr), "reason": choice, "note": note, "asset": [{"id": asset_uuid}]}]
            request_data('POST', '/api/v2/assets/bulk-jobs/acr', payload=lumin_payload)

        if not lumin_list:
            click.echo("\nWe did not find a Tag with that Category or Value... "
                       "It's also possible the tag has not been applied to any assets or the navi DB needs to be updated\n")
            click.echo("If you think this is an error, surround your category and value in \"\"\n")
            exit()

    else:
        click.echo("\nYou can't have a score below 1 or higher than 10\n")



from .database import db_query
import click
import csv
import textwrap


@click.command(help="Pull out CVE data into a nice CSV")
@click.argument('uuid')
def compare(uuid):
    data = db_query("select plugin_id, plugin_name, cvss_base_score, cvss3_base_score,  cves, severity, score, "
                    "first_found, last_found from vulns where asset_uuid='{}' and cves !=' ';".format(uuid))

    with open('cve_dump_{}.csv'.format(uuid), mode='w', encoding='utf-8', newline="") as csv_file:
        agent_writer = csv.writer(csv_file, delimiter=',', quotechar='"')

        header_list = ["Plugin ID", "Plugin Name", "CVE", "CVSS", "CVSS3", "VPR Score", "Severity", "First Found",
                       "Last_Found", "Instances"]

        agent_writer.writerow(header_list)

        master_list = []
        click.echo("\n{:10} {:75} {:16} {:6} {:6} {:6} {:10} {}".format("Plugin ID", "Plugin Name", "CVE", "CVSS", "CVSS3", "VPR", "Severity", "instances"))
        click.echo("-" * 150)
        for plugin in data:
            plugin_id = plugin[0]
            plugin_name = str(plugin[1])
            cvss = str(plugin[2])
            cvss3 = str(plugin[3])

            try:
                cve_list = eval(plugin[4])
            except SyntaxError:
                cve_list = ["NO-CVE"]

            severity = plugin[5]
            vpr = str(plugin[6])
            first_found = str(plugin[7])
            last_found = str(plugin[8])

            for cve in cve_list:

                if cve not in master_list:
                    # Count total instances
                    instances = db_query("select count(*) from vulns where cves LIKE '%" + cve + "%';")

                    master_list.append(cve)
                    click.echo("{:10} {:75} {:16} {:6} {:6} {:6} {:10} {}".format(plugin_id, textwrap.shorten(plugin_name, width=65), cve, cvss, cvss3, vpr, severity, instances[0][0]))

                    csv_update_list = [plugin_id, plugin_name, cve, cvss, cvss3, vpr, severity, first_found, last_found, instances[0][0]]
                    agent_writer.writerow(csv_update_list)

    click.echo("\nYou're export: cve_dump_{}.csv is finished\n".format(uuid))



import click
from .api_wrapper import tenb_connection
from .database import new_db_connection, insert_fixed, db_query, drop_tables, create_table
from .dbconfig import create_fixed_table
import dateutil.parser as dp
import arrow
import time


tio = tenb_connection()


def print_sla():
    try:
        sla_data = db_query("select * from sla;")
        click.echo("\nHere is your Current SLA data")

        critical, high, medium, low = sla_data[0]

        click.echo("\n     Critical SLA: {}".format(critical))
        click.echo("     High SLA: {}".format(high))
        click.echo("     Medium SLA: {}".format(medium))
        click.echo("     Low SLA: {}\n".format(low))
    except:
        # on failure, lets set the defaults.
        critical = 7
        high = 14
        medium = 30
        low = 180
        reset_sla(critical, high, medium, low)


def reset_sla(critical, high, medium, low):
    print("\n Resetting your SLA\n")
    database = r"navi.db"
    conn = new_db_connection(database)
    drop_tables(conn, 'sla')

    create_sla_table = """CREATE TABLE IF NOT EXISTS sla (
                                critical text,
                                high text,
                                medium text, 
                                low text 
                                );"""
    create_table(conn, create_sla_table)

    sla_info = (critical, high, medium, low)
    with conn:
        sql = '''INSERT or IGNORE into sla(critical, high, medium, low) VALUES(?,?,?,?)'''
        cur = conn.cursor()
        cur.execute(sql, sla_info)


def check_sla():
    # Check to see if the SLA database has been created; if so, display the sla
    try:
        data = db_query("select * from sla;")
        print_sla()
    except:
        # on failure, lets set the defaults.
        critical = 7
        high = 14
        medium = 30
        low = 180
        reset_sla(critical, high, medium, low)
        print_sla()


def sla_compare(severity, seconds):
    sla_data = db_query("select * from sla;")

    critical, high, medium, low = sla_data[0]

    if severity == "critical":
        if seconds > int(critical) * 86400:
            return "Fail"
        else:
            return "Pass"

    elif severity == "high":

        if seconds > int(high) * 86400:
            return "Fail"
        else:
            return "Pass"

    elif severity == "medium":

        if seconds > int(medium) * 86400:
            return "Fail"
        else:
            return "Pass"

    elif severity == "low":

        if seconds > int(low) * 86400:
            return "Fail"
        else:
            return "Pass"

    else:
        pass


def compare_and_return_delta(last_fixed, first_found):

    if last_fixed is None:
        parsed_first_found = dp.parse(first_found)
        first_found_in_seconds = parsed_first_found.timestamp()

        current_time = time.time()

        new_time = current_time - first_found_in_seconds
    else:
        parsed_first_found = dp.parse(first_found)
        first_found_in_seconds = parsed_first_found.timestamp()

        parsed_last_fixed = dp.parse(last_fixed)
        last_fixed__in_seconds = parsed_last_fixed.timestamp()

        new_time = last_fixed__in_seconds - first_found_in_seconds

    return new_time


def calculate_sla(severity):

    if severity == 'total':
        total = db_query("select count(plugin_id) from fixed where severity !='info';")[0][0]
        pass_total = db_query("select count(plugin_id) from fixed where state=='FIXED' and pass_fail =='Pass';")[0][0]
        fixed_total = db_query("select count(plugin_id) from fixed where state=='FIXED';")[0][0]

        print("\nTotal Vulns found: {}".format(total))
        print("Total vulns fixed in this period: {}".format(fixed_total))
        try:
            print("Total Fixed rate: {:.0%}\n".format((fixed_total/total)))
        except ZeroDivisionError:
            print("Total Fixed rate: 0%")

        print("Total vulns fixed within SLA: {}".format(pass_total))
        try:
            print("{:.0%} Success Rate".format((pass_total/total)))
        except ZeroDivisionError:
            print("0% Success Rate")

    else:
        severity_total = db_query("select count(plugin_id) from fixed where severity =='{}';".format(severity))[0][0]
        sevrity_pass_total = db_query("select count(plugin_id) from fixed where severity =='{}' and pass_fail =='Pass';".format(severity))[0][0]

        print("\nTotal {} Vulns found this period: {}".format(severity, severity_total ))
        print("Total {} vulns fixed within SLA: {}".format(severity, sevrity_pass_total))
        try:
            print("{:.0%} Success Rate".format((sevrity_pass_total/severity_total)))
        except ZeroDivisionError:
            print("0% Success Rate")


def fixed_export(category, value, days):
    check_sla()
    database = r"navi.db"
    fixed_conn = new_db_connection(database)

    drop_tables(fixed_conn, "fixed")

    create_fixed_table()

    click.echo("\n***Navi calculates SLAs on export request.  So re-run 'navi update fixed' after changing your SLA***\n")
    click.echo("Downloading all of your Fixed, Open and Reopened vulns into a new Table called 'fixed'\n")
    with fixed_conn:
        if category and value:
            tags = (category, value)

            for vulns in tio.exports.vulns(state=['fixed', 'open', 'reopened'], tags=[tags], since=int(arrow.now().shift(days=-int(days)).timestamp())):

                asset_uuid = vulns['asset']['uuid']
                port = vulns['port']['port']
                plugin_id = vulns['plugin']['id']
                plugin_name = vulns['plugin']['name']

                first_found = vulns['first_found']
                try:
                    last_fixed = vulns['last_fixed']
                except KeyError:
                    last_fixed = None

                last_found = vulns['last_found']

                try:
                    output = vulns['output']
                except KeyError:
                    output = None

                severity = vulns['severity']
                # Get the delta from today's date or ast fixed from the time it was first found
                delta = compare_and_return_delta(last_fixed, first_found)
                # Compare that data against the current sla
                pass_fail = sla_compare(severity, delta)

                state = vulns['state']

                special_url = "https://cloud.tenable.com/tio/app.html#/vulnerability-management/dashboard/assets/asset-details/{}/vulns/vulnerability-details/{}/details".format(asset_uuid, plugin_id)

                data_list = [asset_uuid, output, plugin_id, plugin_name, port, first_found, last_fixed, last_found, severity, delta, pass_fail, state, special_url]

                insert_fixed(fixed_conn, data_list)

        else:

            for vulns in tio.exports.vulns(state=['fixed', 'open', 'reopened'], since=int(arrow.now().shift(days=-int(days)).timestamp())):

                asset_uuid = vulns['asset']['uuid']
                port = vulns['port']['port']
                plugin_id = vulns['plugin']['id']
                plugin_name = vulns['plugin']['name']

                first_found = vulns['first_found']

                try:
                    last_fixed = vulns['last_fixed']
                except KeyError:
                    last_fixed = None

                last_found = vulns['last_found']

                try:
                    output = vulns['output']
                except KeyError:
                    output = None

                severity = vulns['severity']
                # Get the delta from today's date or ast fixed from the time it was first found
                delta = compare_and_return_delta(last_fixed, first_found)
                # Compare that data against the current sla
                pass_fail = sla_compare(severity, delta)

                state = vulns['state']

                special_url = "https://cloud.tenable.com/tio/app.html#/vulnerability-management/dashboard/assets/asset-details/{}/vulns/vulnerability-details/{}/details".format(asset_uuid, plugin_id)

                data_list = [asset_uuid, output, plugin_id, plugin_name, port, first_found, last_fixed, last_found, severity, delta, pass_fail, state, special_url]

                insert_fixed(fixed_conn, data_list)
    click.echo("Success!")
